<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Models\Chat;
use App\Models\ChatMessage;
use App\Models\User;
use Illuminate\Http\Request;

class ChatController extends Controller
{

    public function create(Request $request)
    {
        $request->validate([
            'local_id' => 'required',
            'tourist_id' => 'required',
        ]);

        // check if chat exists already between this client and lawyer
        $chat = Chat::where('local_id', $request->local_id)->where('tourist_id', $request->tourist_id)->first();
        if (!$chat) {
            // create chat
            $chat = new Chat();
            $chat->local_id = $request->local_id;
            $chat->tourist_id = $request->tourist_id;
            $chat->save();
        }

        // returns chat, chat messages with sender, client, lawyer
        $chat = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->where('local_id', $request->local_id)->where('tourist_id', $request->tourist_id)->first();
        return response()->json([
            'chat' => $chat,
        ]);
    }

    public function get(Request $request)
    {
        $request->validate([
            'chat_id' => 'required'
        ]);
        // check if the chat exists
        $chat = Chat::find($request->chat_id);
        if ($chat) {
            $chat = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->find($chat->id);
            return response()->json([
                'chat' => $chat,
            ]);
            // // check if chat belongs to current user
            // if (Helper::chatBelongsToUser($chat)) {
            //     // returns chat, chat messages with sender, client, lawyer
            //     $chat = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->find($chat->id);
            //     return response()->json([
            //         'chat' => $chat,
            //     ]);
            // }
        }
        return response()->json([
            'message' => 'Chat not found'
        ], 500);
    }

    public function update(Request $request)
    {
        $request->validate([
            'chat_id' => 'required',
        ]);
        // check if the chat exists
        $chat = Chat::find($request->chat_id);
        if ($chat) {
            // update the chat or sent the message
            $chat_message = new ChatMessage();
            $chat_message->message = $request->message;
            $chat_message->user_id = $request->user()->id;
            $chat_message->chat_id = $chat->id;
            $chat_message->save();
            // save file if request has it
            if ($request->file) {
                $file = $request->file('file');
                $destinationPath = 'uploads/chat_files/' . $chat->id;
                $file_uploaded = $file->move($destinationPath, $file->getClientOriginalName());
                $full_unique_path = $file_uploaded->getPath() . '/' . $file_uploaded->getFilename();
                $chat_message->file = asset($full_unique_path);
                $chat_message->save();
            }
            // fire event
//                event(new Message($chat, $chat_message));
            // returns chat, chat messages with sender, client, lawyer
            $chat = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->find($chat->id);
            return response()->json([
                'chat' => $chat,
            ]);
        }

        return response()->json([
            'message' => 'Chat not found'
        ], 500);
    }

    public function index(Request $request)
    {
        $user = $request->user();
        $chats = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->where('tourist_id', $user->id)->orWhere('local_id', $user->id)->get();

        // if (Helper::isUserLocal()) {
        //     $chats = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->where('local_id', $user->id)->get();
        // } else if (Helper::isUserTourist()) {
        //     $chats = Chat::with(['messages', 'messages.sender', 'local', 'tourist'])->where('tourist_id', $user->id)->get();
        // }

        return response()->json([
            'chats' => $chats,
        ]);
    }

}
