<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Models\Offer;
use Illuminate\Http\Request;

class OfferController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        return response()->json([
            'offers' => Offer::all()
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([            
            'image' => 'required',  
            'name' => 'required',   
            'price' => 'required',  
            'description' => 'required',    
            'web_link' => 'required',   
        ]);

        $user = $request->user();
        if(!$user->hasRole(Helper::ADMIN_TYPE)){
            return response()->json([
                'message' => 'Only accessible by admins.'
            ], 500);
        }

        // add profile picture
        $file = $request->file('image');
        $destinationPath = 'uploads/offer_images/'.time().'/';
        $file_uploaded = $file->move($destinationPath, $file->getClientOriginalName());

        $full_unique_path = $file_uploaded->getPath() . '/' . $file_uploaded->getFilename();

        Offer::create([
            'image' => asset($full_unique_path),  
            'name' => $request->name,   
            'price' => $request->price,  
            'description' => $request->description,    
            'web_link' => $request->web_link,   
        ]);

        return response()->json([
            'offers' => Offer::all()
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
