<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Rules\PaymentMethodIfExtendedPlanRule;
use Illuminate\Http\Request;
use App\Services\SubscriptionService;

class PlanController extends Controller
{
    public function setup_on_signup(Request $request)
    {
        $request->validate([
            'plan_id' => ['required', new PaymentMethodIfExtendedPlanRule($request)],
        ]);

        $user = $request->user();

        if ($request->plan_id == Helper::FREE_PLAN) {
            $user->plan_id = Helper::FREE_PLAN;
            $user->save();
            return response()->json([
                'subscription' => 'free'
            ]);
        }
        else if($request->plan_id == Helper::EXTENDED_PLAN){
            $user->plan_id = Helper::EXTENDED_PLAN;
            $user->save();
            $customer = (new SubscriptionService())->createCustomer();
            if ($customer['status']) {
                $params = [
                    'type' => 'card',
                    'card' => [
                        'number' => @$request->input('card')['number'],
                        'exp_month' => @$request->input('card')['exp_month'],
                        'exp_year' => @$request->input('card')['exp_year'],
                        'cvc' => @$request->input('card')['cvc'],
                    ]
                ];
                $payment_method = (new SubscriptionService())->createAndApplyPaymentMethod($params);
                if ($payment_method['status']) {
                    $subscription =  (new SubscriptionService())->subscribe();
                    if($subscription['status']){
                        $user->is_subscription_paid = true;
                        $user->save();
                        return response()->json([
                            'subscription' => $subscription['subscription']
                        ]); 
                    } else {
                        return response()->json([
                            'message' => $subscription['message']
                        ], 500);
                    }
                } else {
                    return response()->json([
                        'message' => $payment_method['message']
                    ], 500);
                }
            }
            else{
                return response()->json([
                    'message' => $customer['message']
                ], 500);
            }
        }else{
            return response()->json([
                'message' => 'Invalid plan selected'
            ], 500);
        }   
    }

    public function setup_payment_for_tourist(Request $request)
    {
        $request->validate([
            'plan_id' => ['required', new PaymentMethodIfExtendedPlanRule($request)],
        ]);

        $user = $request->user();

        if ($request->plan_id == Helper::FREE_PLAN) {
            $user->plan_id_tourist = Helper::FREE_PLAN;
            $user->save();
            return response()->json([
                'subscription' => 'free'
            ]);
        }
        else if($request->plan_id == Helper::EXTENDED_PLAN){
            $user->plan_id = Helper::EXTENDED_PLAN;
            $user->save();
            $customer = (new SubscriptionService())->createCustomer();
            if ($customer['status']) {
                $params = [
                    'type' => 'card',
                    'card' => [
                        'number' => @$request->input('card')['number'],
                        'exp_month' => @$request->input('card')['exp_month'],
                        'exp_year' => @$request->input('card')['exp_year'],
                        'cvc' => @$request->input('card')['cvc'],
                    ]
                ];
                $payment_method = (new SubscriptionService())->createAndApplyPaymentMethod($params);
                if ($payment_method['status']) {
                    $subscription =  (new SubscriptionService())->subscribe_tourist();
                    if($subscription['status']){
                        $user->is_subscription_paid_tourist = true;
                        $user->save();
                        return response()->json([
                            'subscription' => $subscription['subscription']
                        ]); 
                    } else {
                        return response()->json([
                            'message' => $subscription['message']
                        ], 500);
                    }
                } else {
                    return response()->json([
                        'message' => $payment_method['message']
                    ], 500);
                }
            }
            else{
                return response()->json([
                    'message' => $customer['message']
                ], 500);
            }
        }else{
            return response()->json([
                'message' => 'Invalid plan selected'
            ], 500);
        }   
    }

    public function get_payment_methods(Request $request)
    {
        return response()->json([
            'payment_methods' => (new SubscriptionService())->get_payment_methods()
        ]);
    }

    public function default_payment_method(Request $request)
    {
        return response()->json([
            'default_payment_method' => (new SubscriptionService())->default_payment_method()
        ]);
    }

    public function make_default_payment_method(Request $request)
    {
        $request->validate([
            'payment_method_id' => 'required'
        ]);
        return response()->json([
            'default_payment_method' => (new SubscriptionService())->make_default_payment_method($request->payment_method_id)
        ]);
    }

    public function cancel_subscription(Request $request)
    {
        $res = (new SubscriptionService())->cancel_subscription();
        dd($res);
    }

    public function remove_payment_method(Request $request)
    {
        $request->validate([
            'payment_method_id' => 'required'
        ]);

        $payment_method = (new SubscriptionService())->remove_payment_method($request->payment_method_id);
        if($payment_method['status']){
            return response()->json($payment_method);
        }
        else{
            return response()->json($payment_method, 500);
        }
    }

    public function add_payment_method(Request $request)
    {
        $request->validate([
            'number' => 'required',
            'exp_month' => 'required',
            'exp_year' => 'required',
            'cvc' => 'required',
        ]);

        $params = [
            'type' => 'card',
            'card' => [
                'number' => @$request->input('number'),
                'exp_month' => @$request->input('exp_month'),
                'exp_year' => @$request->input('exp_year'),
                'cvc' => @$request->input('cvc'),
            ]
        ];

        $payment_method = (new SubscriptionService())->createAndApplyPaymentMethod($params);
        if ($payment_method['status']) {
            return response()->json([
                'paymentMethod' => $payment_method['paymentMethod']
            ]);
        } else {
            return response()->json([
                'message' => $payment_method['message']
            ], 500);
        }
        
    }
}
