<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Models\Trip;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use PHPUnit\TextUI\Help;

class TripController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index()
    {
        //
    }

    public function cancel_trip(Request $request)
    {
        $request->validate([
            'trip_id' => 'required'
        ]);

        $trip = Trip::findOrFail($request->trip_id);
        $user = $request->user();

        if ($trip->tourist_id == $user->id) {
            $trip->status = Helper::TRIP_CANCELLED_STATUS;
            $trip->save();
        }

        if ($trip->offers->count() > 0) {
            foreach ($trip->offers as $offer) {
                $offer->status = Helper::TRIP_OFFER_CANCELLED_STATUS;
                $offer->save();
            }
        }

        return response()->json([
            'trip' => Trip::with('local', 'tourist')->where('id', $trip->id)->first(),
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(Request $request)
    {
        $request->validate([
            //            'local_id' => 'required',
            'trip_date' => 'required',
            'trip_time' => 'required',
            'trip_name' => 'required',
            'trip_image' => 'required',
            'duration_in_hours' => 'required',
            'place_id' => 'required',
            'gender' => 'required',
            'tourist_min_budget' => 'required',
            'tourist_max_budget' => 'required',
        ]);

        // check user type
        $user = $request->user();
        $trip = Trip::create([
            //                'local_id' => $local->id,
            'tourist_id' => $user->id,
            'trip_name' => $request->trip_name,
            'trip_date' => date("Y-m-d H:i:s", substr($request->trip_date, 0, 10)),
            'trip_time' => date("Y-m-d H:i:s", substr($request->trip_time, 0, 10)),
            'duration_in_hours' => $request->duration_in_hours,
            'place_id' => $request->place_id,
            'gender' => $request->gender,
            'tourist_min_budget' => $request->tourist_min_budget,
            'tourist_max_budget' => $request->tourist_max_budget,
            'status' => Helper::TRIP_PENDING_STATUS,
        ]);

        // save file if request has it
        if ($request->trip_image) {
            $file = $request->file('trip_image');
            $destinationPath = 'uploads/trip_images/' . $trip->id;
            $file_uploaded = $file->move($destinationPath, $file->getClientOriginalName());
            $full_unique_path = $file_uploaded->getPath() . '/' . $file_uploaded->getFilename();
            $trip->trip_image = asset($full_unique_path);
            $trip->save();
        }

        return response()->json([
            'trip' => Trip::with('local', 'tourist')->where('id', $trip->id)->first(),
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return Response
     */
    public function destroy($id)
    {
        //
    }

    /**
     * @param Request $request
     * @param Trip $trip
     * @return JsonResponse
     */
    public function accept(Request $request, Trip $trip): JsonResponse
    {

        $user = $request->user();

        if (Helper::isUserLocal()) {
            if ($trip->local_id == $user->id) {
                if ($trip->status == Helper::TRIP_PENDING_STATUS) {
                    $trip->accepted_by_local = true;
                    $trip->status = Helper::TRIP_IN_PROGRESS_STATUS;
                    $trip->save();
                    return response()->json([
                        'trip' => Trip::with('local', 'tourist')->where('id', $trip->id)->first(),
                    ]);
                }
            }
        }

        return response()->json([
            'message' => 'Trip not found',
        ], 500);
    }

    /**
     * @param Request $request
     * @param Trip $trip
     * @return JsonResponse
     */
    public function reject(Request $request, Trip $trip): JsonResponse
    {

        $user = $request->user();

        if (Helper::isUserLocal()) {
            if ($trip->local_id == $user->id) {
                if ($trip->status == Helper::TRIP_PENDING_STATUS) {
                    $trip->accepted_by_local = false;
                    $trip->status = Helper::TRIP_CANCELLED_STATUS;
                    $trip->save();
                    return response()->json([
                        'trip' => Trip::with('local', 'tourist')->where('id', $trip->id)->first(),
                    ]);
                }
            }
        }

        if (Helper::isUserTourist()) {
            if ($trip->tourist_id == $user->id) {
                if ($trip->status == Helper::TRIP_PENDING_STATUS) {
                    $trip->delete();
                    return response()->json([
                        'message' => 'Trip is removed.',
                    ]);
                }
            }
        }

        return response()->json([
            'message' => 'Trip not found',
        ], 500);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function approved_trips(Request $request): JsonResponse
    {
        $user = $request->user();

        if (Helper::isUserLocal()) {
            return response()->json([
                'trips' => Trip::with('local', 'tourist')->where('local_id', $user->id)->where('status', Helper::TRIP_IN_PROGRESS_STATUS)->get(),
            ]);
        }

        if (Helper::isUserTourist()) {
            return response()->json([
                'trips' => Trip::with('local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_IN_PROGRESS_STATUS)->get(),
            ]);
        }

        return response()->json([
            'message' => 'You are not local',
        ], 500);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function pending_trips(Request $request): JsonResponse
    {
        return response()->json([
            'trips' => Trip::with('offers', 'local', 'tourist')->where('status', Helper::TRIP_PENDING_STATUS)->get(),
        ]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function user_pending_trips(Request $request): JsonResponse
    {
        $user = $request->user();
        if (Helper::isUserLocal()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('local_id', $user->id)->where('status', Helper::TRIP_PENDING_STATUS)->get(),
            ]);
        } else if (Helper::isUserTourist()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_PENDING_STATUS)->get(),
            ]);
        } else {
            return response()->json([
                'trips' => []
            ]);
        }
    }

    /**
     * @return JsonResponse
     */
    public function in_progress_trips(Request $request): JsonResponse
    {
        $user = $request->user();
        if (Helper::isUserLocal()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('local_id', $user->id)->where('status', Helper::TRIP_IN_PROGRESS_STATUS)->get(),
            ]);
        } else if (Helper::isUserTourist()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_IN_PROGRESS_STATUS)->get(),
            ]);
        } else {
            return response()->json([
                'trips' => []
            ]);
        }
    }

    /**
     * @return JsonResponse
     */
    public function cancelled_trips(Request $request): JsonResponse
    {
        $user = $request->user();
        if (Helper::isUserLocal()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('local_id', $user->id)->where('status', Helper::TRIP_CANCELLED_STATUS)->get(),
            ]);
        } else if (Helper::isUserTourist()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_CANCELLED_STATUS)->get(),
            ]);
        } else {
            return response()->json([
                'trips' => []
            ]);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function completed_trips(Request $request): JsonResponse
    {
        $user = $request->user();

        if (Helper::isUserLocal()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('local_id', $user->id)->where('status', Helper::TRIP_COMPLETED_STATUS)->get(),
            ]);
        }

        if (Helper::isUserTourist()) {
            return response()->json([
                'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_COMPLETED_STATUS)->get(),
            ]);
        }

        return response()->json([
            'message' => 'Trips not found.',
        ], 500);
    }

    public function mark_complete(Request $request)
    {
        $request->validate([
            'trip_id' => 'required'
        ]);
        $user = $request->user();
        $trip = Trip::find($request->trip_id);
        if ($trip) {
            $message = '';
            if ($trip->local_id == $user->id) {
                // mark complete by local
                $trip->marked_finished_by_local = true;
                $trip->save();
                $message = 'Marked completed successfully by local';
            } else if ($trip->tourist_id == $user->id) {
                // mark complete by tourist
                $trip->marked_finished_by_tourist = true;
                $trip->save();
                $message = 'Marked completed successfully by tourist';
            }
            if ($trip->marked_finished_by_local && $trip->marked_finished_by_tourist) {
                $trip->status = Helper::TRIP_COMPLETED_STATUS;
                $trip->save();

                $message = 'Trip Marked completed successfully';
            }
            return response()->json([
                'message' => $message
            ]);
        }
        return response()->json([
            'message' => 'Trip not found',
        ], 500);
    }

    public function local_pending(Request $request)
    {
        $user = $request->user();
        return response()->json([
            'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_PENDING_STATUS)->get(),
        ]);
    }

    public function local_in_progress(Request $request)
    {
        $user = $request->user();
        return response()->json([
            'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_IN_PROGRESS_STATUS)->get(),
        ]);
    }

    public function local_cancelled(Request $request)
    {
        $user = $request->user();
        return response()->json([
            'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_CANCELLED_STATUS)->get(),
        ]);
    }

    public function local_completed(Request $request)
    {
        $user = $request->user();
        return response()->json([
            'trips' => Trip::with('offers', 'local', 'tourist')->where('tourist_id', $user->id)->where('status', Helper::TRIP_COMPLETED_STATUS)->get(),
        ]);
    }
}
