<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Models\Trip;
use App\Models\TripOffer;
use Illuminate\Http\Request;

class TripOfferController extends Controller
{
    public function create(Request $request)
    {
        $request->validate([
            'trip_id' => 'required',
            'offer_amount' => 'required',
        ]);

        $trip_offer = new TripOffer();
        if (Helper::isUserLocal()){
            $trip_offer->fill($request->all());
            $trip_offer->status = Helper::TRIP_OFFER_PENDING_STATUS;
            $trip_offer->local_id = $request->user()->id;
            $trip_offer->save();
        }

        return response()->json([
            'trip_offer' => TripOffer::with('local', 'trip')->where('id', $trip_offer->id)->first(),
        ]);

    }

    public function sent_offers(Request $request)
    {
        $user = $request->user();
        return response()->json([
            'trip_offers' => TripOffer::with('local', 'trip')->where('local_id', $user->id)->get()
        ]);
    }

    public function offers_by_trip(Request $request)
    {
        $request->validate([
            'trip_id' => 'required'
        ]);

        return response()->json([
            'trip_offers' => TripOffer::with('local', 'trip')->where('trip_id', $request->trip_id)->get()
        ]);

    }

    public function accept_offer(Request $request)
    {
        $request->validate([
            'offer_id' => 'required'
        ]);

        try {
            $trip_offer = TripOffer::findOrFail($request->offer_id);
            $trip = Trip::findOrFail($trip_offer->trip_id);
            if ($trip->status == Helper::TRIP_PENDING_STATUS){
                $trip_offer->status = Helper::TRIP_OFFER_ACCEPTED_STATUS;
                $trip_offer->save();

                // change the status of trip to in progress
                $trip->status = Helper::TRIP_IN_PROGRESS_STATUS;
                $trip->local_id = $trip_offer->local_id;
                $trip->save();
            }

            return response()->json([
                'trip_offer' => TripOffer::with('local', 'trip')->where('id', $trip_offer->id)->first()
            ]);
        }catch (\Exception $exception){
            return response()->json([
                'message' => $exception->getMessage()
            ]);
        }

    }

    public function cancel_offer(Request $request)
    {
        $request->validate([
            'offer_id' => 'required'
        ]);

        try {
            $trip_offer = TripOffer::findOrFail($request->offer_id);
            $trip = Trip::findOrFail($trip_offer->trip_id);

            if ($trip->status == Helper::TRIP_PENDING_STATUS){

                $trip_offer->status = Helper::TRIP_OFFER_CANCELLED_STATUS;
                $trip_offer->save();

                // change the status of trip to in progress
                $trip->status = Helper::TRIP_PENDING_STATUS;
                $trip->save();

            }

            return response()->json([
                'trip_offer' => TripOffer::with('local', 'trip')->where('id', $trip_offer->id)->first()
            ]);
        }catch (\Exception $exception){
            return response()->json([
                'message' => $exception->getMessage()
            ]);
        }


    }










}
