<?php

namespace App\Services;

use App\Helper\Helper;
use Auth;
use Exception;
use Laravel\Cashier\Exceptions\InvalidCustomer;
use Stripe\Exception\AuthenticationException;
use Stripe\StripeClient;

class SubscriptionService
{
    public function __construct()
    {
    }

    public function createAndApplyPaymentMethod($params)
    {
        $stripe = (new StripeClient(env('STRIPE_SECRET')));
        try{
            $payment_method = $stripe->paymentMethods->create($params)->toArray();
            $created_payment_method = $stripe->paymentMethods->retrieve(
                $payment_method['id'],
                []
            )->toArray();

            foreach ($this->get_payment_methods() as $existing_payment_method) {
                if ($existing_payment_method['card']['fingerprint'] == $created_payment_method['card']['fingerprint']) {
                    $payment_method = $existing_payment_method;
                    break;
                }
            }
            $paymentMethod = $this->add_payment_method($payment_method['id']);
            if($paymentMethod){
                return [
                    'status' => true,
                    'paymentMethod' => $paymentMethod
                ];
            }
            return [
                'status' => false,
                'message' => 'Payment method can not be added to this user\'s account'
            ];
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
    }

    public function get_payment_methods()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        $paymentMethods = $user->paymentMethods();
        return $paymentMethods->toArray();
    }

    public function default_payment_method()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        $paymentMethod = $user->defaultPaymentMethod();
        return $paymentMethod->toArray();
    }

    public function setup_payment_intent()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        dd($user->createSetupIntent());
        return view('payment', [
            'intent' => $user->createSetupIntent(),
        ]);
        // return $user->createSetupIntent()->toArray();
    }

    public function add_payment_method($payment_method_id)
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        try{
            $user->addPaymentMethod($payment_method_id);
            $paymentMethod = $user->findPaymentMethod($payment_method_id);
            $user->updateDefaultPaymentMethod($payment_method_id);

            return $paymentMethod->toArray();
        }catch(Exception $exception){
            return false;
        }
    }

    public function subscribe()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        try{
            if ($user->hasDefaultPaymentMethod()) {
                $paymentMethod = $user->defaultPaymentMethod();
                $subscription = $user->newSubscription('extended', Helper::STRIPE_PLAN_300_SAR)->create($paymentMethod->id)->toArray();
                $stripe = (new StripeClient(env('STRIPE_SECRET')));
                $price = $stripe->prices->retrieve(
                    @$subscription['stripe_price'],
                    []
                );
                $subscription['price'] = $price;
                return [
                    'status' => true,
                    'subscription' => $subscription
                ];
            }
            return [
                'status' => false,
                'message' => 'Payment method not found.'
            ];
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
    }

    public function subscribe_tourist()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        try{
            if ($user->hasDefaultPaymentMethod()) {
                $paymentMethod = $user->defaultPaymentMethod();
                $subscription = $user->newSubscription('extended_tourist', Helper::STRIPE_PLAN_50_SAR_TOURIST)->create($paymentMethod->id)->toArray();
                $stripe = (new StripeClient(env('STRIPE_SECRET')));
                $price = $stripe->prices->retrieve(
                    @$subscription['stripe_price'],
                    []
                );
                $subscription['price'] = $price;
                return [
                    'status' => true,
                    'subscription' => $subscription
                ];
            }
            return [
                'status' => false,
                'message' => 'Payment method not found.'
            ];
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
    }

    public function createCustomer()
    {
        // $user = Auth::loginUsingId(6); // login as
        $user = Auth::user(); // login as
        try {
            $customer = $user->asStripeCustomer();
            return [
                'status' => true,
                'customer' => $customer
            ];
        } catch (InvalidCustomer $exception) {
            try {
                $customer = $user->createAsStripeCustomer();
                return [
                    'status' => true,
                    'customer' => $customer
                ];
            } catch (AuthenticationException $exception) {
                return [
                    'status' => false,
                    'message' => $exception->getMessage()
                ];
            }
        } catch (AuthenticationException $exception) {
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
    }

    public function remove_payment_method($payment_method_id)
    {
        $user = Auth::user(); // login as
        try{
            $paymentMethod = $user->findPaymentMethod($payment_method_id);
            if ($paymentMethod) {
                $user->deletePaymentMethod($paymentMethod->id);
                return [
                    'status' => true,
                    'message' => 'Payment Method Deleted'
                ];
            }
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
        
    }

    public function make_default_payment_method($payment_method_id)
    {
        $user = Auth::user(); // login as
        try{
            $paymentMethod = $user->findPaymentMethod($payment_method_id);
            if ($paymentMethod) {
                $user->updateDefaultPaymentMethod($payment_method_id);
                return [
                    'status' => true,
                    'message' => 'Payment Method Deleted'
                ];
            }
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
        
    }

    public function cancel_subscription()
    {
        $user = Auth::user(); // login as
        try{
            $user->subscription(Helper::EXTENDED_PLAN)->cancel();
            return [
                'status' => true,
                'message' => 'Subscription cancelled'
            ];
        }catch(Exception $exception){
            return [
                'status' => false,
                'message' => $exception->getMessage()
            ];
        }
        
    }
    
}
