<?php

use App\Helper\Helper;
use App\Http\Controllers\ChatController;
use App\Http\Controllers\OfferController;
use App\Http\Controllers\PlanController;
use App\Http\Controllers\TripController;
use App\Http\Controllers\TripOfferController;
use App\Http\Controllers\UserRatingController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\UserController;
use Laravel\Sanctum\PersonalAccessToken;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
    return $request->user();
});


Route::middleware('auth:sanctum')->post('/trips', [TripController::class, 'store']);

//Route::middleware('auth:sanctum')->post('/trips/{trip}/accept', [TripController::class, 'accept']);
//Route::middleware('auth:sanctum')->post('/trips/{trip}/reject', [TripController::class, 'reject']);
//Route::middleware('auth:sanctum')->get('/trips/approved', [TripController::class, 'approved_trips']);
//Route::middleware('auth:sanctum')->get('/trips/pending', [TripController::class, 'pending_trips']);
//Route::middleware('auth:sanctum')->get('/trips/cancelled', [TripController::class, 'cancelled_trips']);

Route::middleware('auth:sanctum')->get('/trips/pending', [TripController::class, 'pending_trips']);

Route::middleware('auth:sanctum')->get('/tourist/pending', [TripController::class, 'user_pending_trips']);
Route::middleware('auth:sanctum')->get('/tourist/in_progress', [TripController::class, 'in_progress_trips']);
Route::middleware('auth:sanctum')->get('/tourist/cancelled', [TripController::class, 'cancelled_trips']);
Route::middleware('auth:sanctum')->get('/tourist/completed', [TripController::class, 'completed_trips']);

Route::middleware('auth:sanctum')->get('/local/pending', [TripController::class, 'local_pending']);
Route::middleware('auth:sanctum')->get('/local/in_progress', [TripController::class, 'local_in_progress']);
Route::middleware('auth:sanctum')->get('/local/cancelled', [TripController::class, 'local_cancelled']);
Route::middleware('auth:sanctum')->get('/local/completed', [TripController::class, 'local_completed']);

Route::middleware('auth:sanctum')->post('/trip/mark_complete', [TripController::class, 'mark_complete']);

Route::middleware('auth:sanctum')->post('/trips/cancel', [TripController::class, 'cancel_trip']);

Route::middleware('auth:sanctum')->post( '/setup_payment_and_plan', [PlanController::class, 'setup_on_signup']);
Route::middleware('auth:sanctum')->post( '/setup_plan_for_tourist', [PlanController::class, 'setup_payment_for_tourist']);
Route::middleware('auth:sanctum')->get('/payment_methods', [PlanController::class, 'get_payment_methods']);
Route::middleware('auth:sanctum')->post('/remove_payment_method', [PlanController::class, 'remove_payment_method']);
Route::middleware('auth:sanctum')->get('/default_payment_method', [PlanController::class, 'default_payment_method']);
Route::middleware('auth:sanctum')->post('/make_default_payment_method', [PlanController::class, 'make_default_payment_method']);
Route::middleware('auth:sanctum')->post('/add_payment_method', [PlanController::class, 'add_payment_method']);
Route::middleware('auth:sanctum')->post('/cancel_subscription', [PlanController::class, 'cancel_subscription']);

Route::middleware('auth:sanctum')->post('/trip_offers', [TripOfferController::class, 'create']);
Route::middleware('auth:sanctum')->get('/sent_offers', [TripOfferController::class, 'sent_offers']);
Route::middleware('auth:sanctum')->get('/offers_by_trip', [TripOfferController::class, 'offers_by_trip']);
Route::middleware('auth:sanctum')->post('/offers/accept', [TripOfferController::class, 'accept_offer']);
Route::middleware('auth:sanctum')->post('/offers/reject', [TripOfferController::class, 'cancel_offer']);


Route::middleware('auth:sanctum')->get('/locals', [UserController::class, 'locals']);
Route::middleware('auth:sanctum')->get('/tourists', [UserController::class, 'tourists']);
Route::middleware('auth:sanctum')->post('locals/filter', [UserController::class, 'filter_locals']);
Route::middleware('auth:sanctum')->post('locals/search', [UserController::class, 'search_locals']);

Route::middleware('auth:sanctum')->get('/offers', [OfferController::class, 'index']);
Route::middleware('auth:sanctum')->post('/offers', [OfferController::class, 'store']);

Route::middleware('auth:sanctum')->get('settings/activities', [UserController::class, 'activities_get_all']);
Route::middleware('auth:sanctum')->get('settings/locations', [UserController::class, 'locations_get_all']);
Route::middleware('auth:sanctum')->get('settings/languages', [UserController::class, 'languages_get_all']);
Route::middleware('auth:sanctum')->get('settings/currencies', [UserController::class, 'currencies_get_all']);
Route::middleware('auth:sanctum')->get('settings/genders', [UserController::class, 'genders_get_all']);

Route::middleware('auth:sanctum')->get('settings/general', [UserController::class, 'get_user_general_settings']);
Route::middleware('auth:sanctum')->post('settings/general', [UserController::class, 'update_user_general_settings']);
Route::middleware('auth:sanctum')->post('settings/role', [UserController::class, 'update_user_role']);

// create chat or return chat if created
Route::middleware('auth:sanctum')->post('/chat/create', [ChatController::class, 'create']);
// get a specific chat
Route::middleware('auth:sanctum')->get('/chat/get', [ChatController::class, 'get']);
// send a message or update chat
Route::middleware('auth:sanctum')->post('/chat/update', [ChatController::class, 'update']);
// get all chats of user
Route::middleware('auth:sanctum')->get('/chats', [ChatController::class, 'index']);

//
Route::middleware('auth:sanctum')->get('/ratings/{user}', [UserRatingController::class, 'getRatings']);
Route::middleware('auth:sanctum')->post('/ratings', [UserRatingController::class, 'giveRating']);

Route::middleware('auth:sanctum')->post('/admin/delete_local', [UserController::class, 'deleteLocal']);


Route::post('sign_up', [UserController::class, 'signUp']);

Route::post('login', function (Request $request) {
    $login = Auth::attempt([
        'email' => $request->email,
        'password' => $request->password
    ]);
    if ($login) {
        $user = auth()->user();
        return response()->json([
            'user' => $user->with('roles')->where('id', Auth::id())->first(),
            'token' => $user->createToken('mobile_app')->plainTextToken
        ]);
    }

    return response()->json([
        'message' => "Incorrect email and password"
    ], 500);
});



Route::middleware('auth:sanctum')->post('/cancel_subscription', function(Request $request){
    $user = $request->user();
    
    if ($user->subscribed('default')) {
        $user->subscription('default')->cancel();
        
        
        return response()->json([
            'message' => 'Cancelled subscription successfully, moved to Free Plan.',
        ]);
    }
    else{
        return response()->json([
            'message' => 'User has no subscription or already cancelled.',
        ], 500);
    }
    
});
